%
% ausgabeHTML.pl
%
% Ausgabe eines fertigen Plans in HTML
%

allgemeineFunktionalitaeten('allgemeineFunktionalitaeten.pl').

/***********************************************************/
/* ausgabeHTML(+Datei).                                    */
/***********************************************************/
/* Ausgeben des fertigen Plans in HTML.                    */
/***********************************************************/
ausgabeHTML('Plan.html').

/***********************************************************/
/* schreibeHTML(+Stammdaten)                               */
/***********************************************************/
/* Steuern des Ablaufs beim Schreiben der HTML-Ausgabe     */
/***********************************************************/
schreibeHTML(Stammdaten) :-
	allgemeineFunktionalitaeten(AllgemeineFunktionalitaeten),
	compile(AllgemeineFunktionalitaeten),

	writeln("Erzeugen des HTML-Outputs ..."),
	writeln("Einlesen der Stammdaten ..."),
	compile(Stammdaten),
	writeln("Einlesen der berechneten Lsung ..."),
	ausgabeDatei(Loesung),
	compile(Loesung),
	writeln("Sortieren der Lsung ..."),
	erstelleTagListe(TagListe),
	erstelleZeitListe(ZeitListe),
	erstelleRaumListe(RaumListe),
	erstelleTagRaumListe(TagListe, RaumListe, ZeitListe, TagRaumListe),
	writeln("Schreiben der HTMLDatei ..."),
	ausgabeHTML(HTMLDatei),
	initHTMLDatei(HTMLDatei),
	schreibeTabellenGeruest(HTMLDatei),
	writeln("    Schreiben des Plans ..."),
	schreibePlanHTML(HTMLDatei, TagListe, RaumListe, TagRaumListe, ZeitListe),
	schliesseTabellenGeruest(HTMLDatei),
	schliesseHTMLDatei(HTMLDatei),
	writeln("Heureka!").

/***********************************************************/

/***********************************************************/
/* initHTMLDatei(?ZielDatei)                               */
/***********************************************************/
/* Initialisierung der Ausgabedatei                        */                  
/***********************************************************/
initHTMLDatei(Ziel) :-
	open(Ziel, write, Stream),
	print(Stream, "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">"),
	nl(Stream),
	print(Stream, '<html>'),
	nl(Stream),
	print(Stream, '<title>'),
	nl(Stream),
	print(Stream, 'Stundenplan der FH Wedel'),
	nl(Stream),
	print(Stream, '</title>'),
	nl(Stream),
	print(Stream, '<body bgcolor=\"#FFFFFF\">'),
	nl(Stream),
	close(Stream).

/***********************************************************/
/* schliesseHTMLDatei(?ZielDatei)                          */
/***********************************************************/
/* Schliessen der Ausgabedatei                             */
/***********************************************************/
schliesseHTMLDatei(Ziel) :-
	open(Ziel, append, Stream),
	nl(Stream),
	print(Stream, '</body>'),
	nl(Stream),
	print(Stream, '</html>'),
	nl(Stream),
	close(Stream).

/***********************************************************/

/***********************************************************/
/* schreibeTabellenGeruest(?ZielDatei)                     */
/***********************************************************/
/* Anlegen der Haupttabelle                                */
/***********************************************************/
schreibeTabellenGeruest(Ziel) :-
	open(Ziel, append, Stream),
	print(Stream, '<table summary=\"Vorlesungsplan FH Wedel\" border=\"1\" bgcolor=\"#FFFFE0\" cellpadding=\"1\" cellspacing=\"0\">'),
	nl(Stream),
	close(Stream).

/***********************************************************/
/* schliesseTabellenGeruest(?ZielDatei)                    */
/***********************************************************/
/* Schliessen der Haupttabelle                             */
/***********************************************************/
schliesseTabellenGeruest(Ziel) :-
	open(Ziel, append, Stream),
	nl(Stream),
	print(Stream, '</table>'),
	close(Stream).

/***********************************************************/

/***********************************************************/
/* schreibeZeitenHTML(+ZeitListe, ?Stream)                 */
/***********************************************************/
/* Schreiben der Vorlsungszeiten und weiteren              */
/* Spaltenueberschriften                                   */
/***********************************************************/
schreibeZeitenHTML(ZeitListe, Stream) :-
	nl(Stream),
	print(Stream, '<tr bgcolor=\"#C0C0C0\">'),
	nl(Stream),
	print(Stream, '  <th><font size=\"-1\">Tag</font></th>'),
	nl(Stream),
	print(Stream, '  <th><font size=\"-1\">Raum</font></th>'),
	nl(Stream),
	schreibeZeitenHTML1(ZeitListe, Stream),
	print(Stream, '  <th><font size=\"-1\">Raum</font></th>'),
	nl(Stream),
	schliesseTabellenZeile(Stream).

/***********************************************************/
/* schreibeZeitenHTML1(+ZeitListe, ?Stream)                */
/***********************************************************/
/* Schreiben der Zeitelemente                              */
/***********************************************************/
schreibeZeitenHTML1([], _).
schreibeZeitenHTML1([Kopf|Rest], Stream) :-
	print(Stream, '  <th width=\"40%\"><font size=\"-1\">'),
	print(Stream, Kopf),
	print(Stream, '</font></th>'),
	nl(Stream),
	schreibeZeitenHTML1(Rest, Stream).

/***********************************************************/

/***********************************************************/
/* schreibePlanHTML(?ZielDatei, +TagListe, +RaumListe,     */
/*      +TagRaumListe, +ZeitListe)                         */
/***********************************************************/
/* Ablaufsteuerung des Planschreibens                      */
/***********************************************************/
schreibePlanHTML(Ziel, [TagKopf|TagListe], [RaumKopf|RaumListe], [TagRaumKopf|TagRaumListe], [ZeitKopf|ZeitListe]) :-
	open(Ziel, append, Stream),
	schreibePlanHTML1([TagKopf|TagListe], [RaumKopf|RaumListe], [TagRaumKopf|TagRaumListe], [ZeitKopf|ZeitListe], Stream), 
	% am Ende nochmal die Zeiten anfuegen
	schreibeZeitenHTML([ZeitKopf|ZeitListe], Stream),
	close(Stream).

/***********************************************************/
/* schreibePlanHTML1(+TagListe, +RaumListe, +TagRaumListe, */
/*     +ZeitListe, ?Stream)                                */
/***********************************************************/
/* Schreiben der Vorlesungstage und des jeweils ersten     */
/* Raumes                                                  */
/***********************************************************/
schreibePlanHTML1([], _, _, _, _).
schreibePlanHTML1([TagKopf|TagListe], [RaumKopf|RaumListe], TagRaumListe, ZeitListe, Stream) :-
	schreibeZeitenHTML(ZeitListe, Stream),
	schreibeWochentagHTML(TagKopf, [RaumKopf|RaumListe], Stream),
	schreibeRaumHTML(RaumKopf, Stream),
	schreibePlanHTML3([TagKopf|TagListe], [RaumKopf|RaumListe], TagRaumListe, ZeitListe, Stream).

/***********************************************************/
/* schreibePlanHTML2(+TagListe, +RaumListe, +TagRaumListe, */
/*     +ZeitListe, ?Stream)                                */
/***********************************************************/
/* Schreiben der Raeume (ausser 1. Raum)                   */
/***********************************************************/
schreibePlanHTML2(TagListe, [RaumKopf|RaumListe], TagRaumListe, ZeitListe, Stream) :-
	schreibeTabellenZeile(Stream),
	schreibeRaumHTML(RaumKopf, Stream),
	schreibePlanHTML3(TagListe, [RaumKopf|RaumListe], TagRaumListe, ZeitListe, Stream).

/***********************************************************/
/* schreibePlanHTML3(+TagListe, +RaumListe, +TagRaumListe, */
/*     +ZeitListe, ?Stream)                                */
/***********************************************************/
/* Schreiben der Vorlesungen                               */
/*    /1 - letzte Vorlesung eines Tages (Verwaltung)       */
/*    /2 - letzte Vorlesung eines Raumes (Verwaltung)      */
/*    /3 - letzte Vorlesung eines Raumes (plus Zeilenende) */
/*    /4 - ohne Einschraenkungen                           */
/***********************************************************/
schreibePlanHTML3([_|TagListe], [_], TagRaumListe, [], Stream) :-
	erstelleRaumListe(RaumListe),
	erstelleZeitListe(ZeitListe),
	schreibePlanHTML1(TagListe, RaumListe, TagRaumListe, ZeitListe, Stream).
schreibePlanHTML3(TagListe, [_|RaumListe], TagRaumListe, [], Stream) :-
	erstelleZeitListe(ZeitListe),
	schreibePlanHTML2(TagListe, RaumListe, TagRaumListe, ZeitListe, Stream).
schreibePlanHTML3(TagListe, [RaumKopf|RaumListe], [TagRaumKopf|TagRaumListe], [_], Stream) :-
	schreibeVeranstaltungHTML(TagRaumKopf, Stream),
	schreibeRaumHTML(RaumKopf, Stream),
	schliesseTabellenZeile(Stream),
	schreibePlanHTML3(TagListe, [RaumKopf|RaumListe], TagRaumListe, [], Stream).
schreibePlanHTML3(TagListe, RaumListe, [TagRaumKopf|TagRaumListe], [_|ZeitListe], Stream) :-
	schreibeVeranstaltungHTML(TagRaumKopf, Stream),
	schreibePlanHTML3(TagListe, RaumListe, TagRaumListe, ZeitListe, Stream).

/***********************************************************/

/***********************************************************/
/* schreibeWochentagEndeHTML(+Tag, +RaumListe, ?Stream)    */
/***********************************************************/
/* Schliessen des Wochentag-Tags.                          */
/***********************************************************/
schreibeWochentagEndeHTML(Tag, RaumListe, Stream) :-
	length(RaumListe, N),
	print(Stream, '  <td rowspan=\"'),
	print(Stream, N),
	print(Stream, '\" align=\"center\" bgcolor=\"#808080\"><font size=\"-1\"><B>'),
	print(Stream, Tag),
	print(Stream, '</B></font></td>'),
	nl(Stream).

/***********************************************************/
/* schreibeWochentagHTML(+Tag, +Raum, ?Stream)             */
/***********************************************************/
/* Schreiben eines Wochentages                             */
/***********************************************************/
schreibeWochentagHTML(Tag, RaumListe, Stream) :-
	length(RaumListe, N),
	schreibeTabellenZeile(Stream),
	print(Stream, '  <th rowspan=\"'),
	print(Stream, N),
	print(Stream, '\" align=\"center\" bgcolor=\"#808080\"><font size=\"-1\">'),
	print(Stream, Tag),
	print(Stream, '</font></th>'),
	nl(Stream).

/***********************************************************/
/* schreibeRaumHTML(+Raum, ?Stream)                        */
/***********************************************************/
/* Schreiben eines Raumes                                  */
/***********************************************************/
schreibeRaumHTML(Raum, Stream) :-
	print(Stream, '  <td align=\"center\" bgcolor=\"#C0C0C0\" height=\"30\"><font size=\"-2\"><B>'),
	print(Stream, Raum),
	print(Stream, '</B></font></td>'),
	nl(Stream).

/***********************************************************/
/* schreibeVeranstaltungHTML(+ListenElement, ?Stream)      */
/***********************************************************/
/* Schreiben einer Veranstaltung                           */
/***********************************************************/
schreibeVeranstaltungHTML([], Stream) :-
	print(Stream, '  <td bgcolor=\"#F0D306\">&nbsp;</td>'),
	nl(Stream).
schreibeVeranstaltungHTML([(Veranstaltung, Teilnehmer, _, _, _)], Stream) :-
	print(Stream, '  <td colspan=\"1\">'),
	nl(Stream),
	print(Stream, '<table summary=\"Vorlesung\" cellpadding=\"1\" cellspacing=\"0\">'),
	nl(Stream),
	schreibeTabellenZeile(Stream),
	print(Stream, '  <td align=\"left\" valign=\"top\"><font size=\"-2\">'),
	print(Stream, Veranstaltung),
	print(Stream, '</font></td>'),
	nl(Stream),
	schliesseTabellenZeile(Stream),
	schreibeTabellenZeile(Stream),
	print(Stream, '  <td align=\"right\" valign=\"bottom\"><font size=\"-2\">'),    
	print(Stream, Teilnehmer),
	print(Stream, '</font></td>'),
	nl(Stream),
	schliesseTabellenZeile(Stream),
	print(Stream, '</table>'),
	nl(Stream),
	print(Stream, '</td>'),
	nl(Stream).

/***********************************************************/   

/***********************************************************/
/* schreibeTabellenZeile(?Stream)                          */
/***********************************************************/
/* Schreiben einer TabellenZeile                           */
/***********************************************************/
schreibeTabellenZeile(Stream) :-
	print(Stream, '<tr>'),
	nl(Stream).

/***********************************************************/
/* schliesseTabellenZeile(?Stream)                         */
/***********************************************************/
/* Schliessen einer TabellenZeile                          */
/***********************************************************/
schliesseTabellenZeile(Stream) :-
	print(Stream, '</tr>'),
	nl(Stream).

/***********************************************************/
/* schreibeLeerzeileHTML(+Liste, ?Stream)                  */
/***********************************************************/
/* Schreiben einer Leerzeile zwischen den einzelnen Tagen  */
/***********************************************************/
schreibeLeerzeileHTML(Liste, Stream) :-
	length(Liste, N),
        % Anzahl der Spalten der Tabelle
	% N = Anzahl Zeiten,
	% 4 = Tag, Raum am Anfang, Raum am Ende, Tag am Ende
	Breite is N + 4,
	print(Stream, "<TR height=\"1\"><TD colspan=\""),
	print(Stream, Breite),
	print(Stream, "\"></TD></TR>"),
	nl(Stream).









