%
% schreibePlan.pl
%
% Ausgabe der Loesung als Prolog-Fakten
%

% letzte Aenderung 19.11.01 mb:
% suche die veranstaltungs-fakten aus den neuen veranstaltungsfakten
% zusammen, da fuer sonderfaelle dummyveranstaltungen erstellt werden 
% muessen

% letzte Aenderung 29.11.01 rs: Ausgabe um terminNachAbsprache 
% erweitert; Untergruppen durch zugehoerige Obergruppen ersetzt

% letzte Aenderung 25.01.02 rs: letzte Aenderung wieder rueckgaengig 
% gemacht

% letzte Aenderung 4.2.02 rs: aendern des Ausgabeformats, so dass Tage
% und Zeiten als Listen ausgegeben werden (wg. gleichem Format von Ein-
% und Ausgabe

/***********************************************************/
/* schreibePlan(?Array)                                    */
/***********************************************************/
/* Ausgabe des vorlaeufigen Plans in die Ausgabedatei      */
/***********************************************************/
schreibePlan(Stundenplan) :-
	writeln("Schreiben der gefundenen Lsung ..."),
	ausgabeDatei(Loesung),
	write("Als Ausgabedatei wurde "),
	write(Loesung),
	writeln(" gewhlt."),
	open(Loesung, write, Stream),
	print(Stream, '% Loesung'),
	nl(Stream),
	schreibePlan(Stundenplan, Stream),
	close(Stream).

/***********************************************************/
/* schreibePlan(?Array, ?Ausgabedatei)                     */
/***********************************************************/
schreibePlan(Stundenplan, Stream) :-
	findall(veranstaltung(VerName, VerTeilnehmer, VerRessourcen), veranstaltung(_, VerName, VerTeilnehmer, VerRessourcen, _), VeranstaltungsListe),
	schreibeVeranstListe(VeranstaltungsListe, Stream),
	schreibeVerwaltung(Stream),
	wandleArrayInSortListe(Stundenplan, SortListe),
	schreibeTerminNachAbsprache(Stream),
	schreibeStundenplan(SortListe, Stream).

/***********************************************************/
/* schreibeVeranstListe(+VeranstListe, ?Ausgabedatei)      */
/***********************************************************/
/* Schreiben der veranstaltung(_, _, _)-Klauseln in die    */
/* Ausgabedatei                                            */
/***********************************************************/
schreibeVeranstListe([], Stream) :-
	nl(Stream).
schreibeVeranstListe([veranstaltung(VerName, VerTeilnehmer, VerRessourcen)|Rest], Stream) :-
	print(Stream, 'veranstaltung('),
	print(Stream, '\''),
	print(Stream, VerName),
	print(Stream, '\', '),
	print(Stream, VerTeilnehmer),
	print(Stream, ', '),
	print(Stream, VerRessourcen),
	print(Stream, ').'),
	nl(Stream),   
	schreibeVeranstListe(Rest, Stream).

/***********************************************************/
/* wandleArrayInSortListe(?Array, -SortListe)              */
/***********************************************************/
/* Sortiere die Veranstaltungen des Arrays                 */
/***********************************************************/
wandleArrayInSortListe(Stundenplan, SortListe) :-
	anzVeranst(Index),
	erstelleSortListe(Stundenplan, Index, [], SortListe).

/***********************************************************/
/* erstelleSortListe(?Array, +Index, +ZwErgebnis           */
/*      -SortListe)                                        */
/***********************************************************/
/* Sortiere nach Tag, Raum Zeit                            */
/***********************************************************/
erstelleSortListe(_, 0, Liste, SortListe) :-
	sort(Liste, NeueListe),
	wandleInWoerter(NeueListe, SortListe).
erstelleSortListe(Stundenplan, IndexAlt, Liste, SortListe) :-
	subscript(Stundenplan, [IndexAlt, 1], Raum),
	subscript(Stundenplan, [IndexAlt, 2], Tag),
	subscript(Stundenplan, [IndexAlt, 3], Zeit),
	append([(Tag, Raum, Zeit, IndexAlt)], Liste, ListeNeu),
	Index is IndexAlt - 1,
	erstelleSortListe(Stundenplan, Index, ListeNeu, SortListe).

/***********************************************************/
/* wandleInWoerter(+VeranstListe, -Ergebnis)               */
/***********************************************************/
/* Transformierung in die korrespondierenden Woerter       */
/***********************************************************/
wandleInWoerter([], []).
wandleInWoerter([(TagNo, RaumNo, ZeitNo, VerNo)|Rest], [VerListeNeu|VerRest]) :-
	findall((Name, Tag, Zeit, Raum), (veranstaltung(VerNo, Name, _, _, _), raum(RaumNo, Raum, _, _, _), tag(TagNo, Tag, _), zeit(ZeitNo, Zeit, _)), VerListeNeu),
	wandleInWoerter(Rest, VerRest).

/***********************************************************/
/* schreibeTerminNachAbsprache(?Stream)                    */
/***********************************************************/
/* Steuerung des Schreibens von terminNachAbsprache.       */
/***********************************************************/
schreibeTerminNachAbsprache(Stream) :-
	findall((Name, Teilnehmer), terminNachAbsprache(Name, Teilnehmer), Liste),
	schreibeTerminNachAbsprache(Liste, Stream).

/***********************************************************/
/* schreibeTerminNachAbsprache(+Liste, ?Stream)            */
/***********************************************************/
schreibeTerminNachAbsprache([], Stream):- 
	nl(Stream).
schreibeTerminNachAbsprache([(Kopf, Teilnehmer)|Rest], Stream) :-
	print(Stream, 'terminNachAbsprache(\''),
	print(Stream, Kopf),
	print(Stream, '\', '),
	print(Stream, Teilnehmer),
	print(Stream, ').'),
	nl(Stream),
	schreibeTerminNachAbsprache(Rest, Stream).
	
/***********************************************************/
/* schreibeStundenplan(+SortListe, ?Ausgabedatei)          */
/***********************************************************/
/* Schreiben des berechneten Stundenplans in die           */
/* Ausgabedatei                                            */
/***********************************************************/
schreibeStundenplan([], _).
schreibeStundenplan([Kopf|Rest], Stream) :-
	print(Stream, 'veranstaltungAmUmIn('),
	schreibeAmUmInElement(Kopf, Stream),
	print(Stream, ').'),
	nl(Stream),
	schreibeStundenplan(Rest, Stream).

/***********************************************************/
/* schreibeAmUmInElement(+AmUmInListe, ?Ausgabedatei)      */
/***********************************************************/
/* Schreibe ein Elemet der veranstaltungAmUmIn-Klausel     */
/* (formatierte Ausgabe)                                   */
/* - als letztes Element wird die Prioritaet eingefuegt    */
/***********************************************************/
schreibeAmUmInElement([(VerName, TagName, ZeitName, RaumName)], Stream) :-
	print(Stream, '\''),
	print(Stream, VerName),
	print(Stream, '\', [\''),
	print(Stream, TagName),
	print(Stream, '\'], [\''),
	print(Stream, ZeitName),
	print(Stream, '\'], '),
	print(Stream, RaumName).

/***********************************************************/
/* schreibeVerwaltung(+Stream)                             */
/***********************************************************/
/* Schreiben von Fakten, die wegen der Gleichheit der      */
/* Ein- und Ausgabedaten benoetigt werden.                 */
/***********************************************************/
schreibeVerwaltung(Stream) :-
	print(Stream, 'doppelstunde(_, _) :- false.'),
	nl(Stream),
	print(Stream, 'dreifachstunde(_, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'vierfachstunde(_, _, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungNichtAmUmIn(_, _, _, _) :- false.'),
	nl(Stream),	
	print(Stream, 'teilnehmerNichtAmUm(_, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungenAmSelbenTag(_, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungenAmSelbenTag(_, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungenAmSelbenTag(_, _, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungenNichtAmSelbenTag(_, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungenNichtAmSelbenTag(_, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungenNichtAmSelbenTag(_, _, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungenNichtZusammenhaengend(_, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungenNichtZusammenhaengend(_, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'terminNachAbsprache(_) :- false.'),
	nl(Stream),
	print(Stream, 'doppelstunde(_, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'dreifachstunde(_, _, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'vierfachstunde(_, _, _, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'teilnehmerNichtAmUm(_, _, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungAmUmIn(_, _, _, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungNichtAmUmIn(_, _, _, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungenAmSelbenTag(_, _, _, _, _) :- false.'),
	nl(Stream),
	print(Stream, 'veranstaltungenNichtAmSelbenTag(_, _, _, _, _) :- false.'),
	nl(Stream).