%
% stundenplangenerierung.pl
%
% Berechnung eines Stundenplans
%

% letzte Aenderung 7.11.01 mb: auswahl der zu benutzenden Loesungssuche

% letzte Aenderung 6.12.01 rs: Die Auswahl der Planungskomponente 
% geschieht jetzt ueber planungsKomponente(Datei)

% letzte Aenderung 4.2.02 rs: Kommentare erweitert

/***********************************************************/
/* Einbinden der FiniteDomains-Bibliothek von Eclipse      */
/***********************************************************/ 
:- use_module(library(fd)).

/***********************************************************/
/* Sourcedateien, die zur Berechnung benoetigt werden      */
/***********************************************************/
zwischenErgebnis('Ergebnis.pl').   % Name der Datei kann beliebig geaendert werden
testModul('tests.pl').
vorverarbeitung('vorverarbeitung.pl').
essentielleEinschraenkungen('essentielleEinschraenkungen.pl').
harteConstraints('harteConstraints.pl').
schreibePlanDatei('schreibePlan.pl').
allgemeineFunktionalitaeten('allgemeineFunktionalitaeten.pl').

/***********************************************************/
/* Hier einfach gewuenschte Planungskomponente einbinden.  */
/***********************************************************/
planungsKomponente('heuristik.pl').
%planungsKomponente('einfacheHeuristik.pl').

/***********************************************************/
/* start(+Stammdaten, +Vorlesungsdaten)                    */
/***********************************************************/
/* Ablaufsteuerung der Stundenplangenerierung              */
/***********************************************************/
start(Stammdaten, Vorlesungsdaten) :-
	allgemeineFunktionalitaeten(AllgemeineFunktionalitaeten),
	compile(AllgemeineFunktionalitaeten),

	writeln("Einlesen der Stammdaten ..."),
	compile(Stammdaten),
	writeln("Einlesen der Vorlesungsdaten ..."),
	compile(Vorlesungsdaten),
	
	% Tests durchfuehren
	writeln("berprfen der Wissensbasis ..."),
	testModul(Tests),
	compile(Tests),
	fuehreTestsDurch,

	% Vorverarbeitung
	writeln("Starten der Vorverarbeitung ..."),
	vorverarbeitung(Vorverarbeitung),
	compile(Vorverarbeitung),
	zwischenErgebnis(Ergebnis),
	vorverarbeitung(Ergebnis, Stundenplan),

	writeln("Berechnen der essentiellen Einschrnkungen ..."),
	essentielleEinschraenkungen(Einschraenkungen),
	compile(Einschraenkungen),
	compile(Ergebnis),
	berechneEssentielleEinschraenkungen(Stundenplan),
	
	writeln("Berechnen der harten Constraints ..."),
	harteConstraints(Constraints),
	compile(Constraints),
	berechneHarteConstraints(Stundenplan),
	
	writeln("Berechnungen erfolgreich."),	
	
	% Stundenplanung
	writeln("Lsungssuche ..."),
	planungsKomponente(Heuristik),
	compile(Heuristik),
	verarbeitung(Stundenplan),
	
	% Ausgabe als Prologfakten 
	writeln("Ausgeben des fertigen Plans ..."),
	schreibePlanDatei(SchreibePlan),
	compile(SchreibePlan),
	schreibePlan(Stundenplan),

	% Fertig :)
	writeln("Heureka!").















